==> core-io-1.2.3-sources.jar: com/couchbase/client/deps/io/netty/util/concurrent/PromiseNotifier.java <==
/*
 * Copyright 2014 The Netty Project
 *
 * The Netty Project licenses this file to you under the Apache License,
 * version 2.0 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at:
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package com.couchbase.client.deps.io.netty.util.concurrent;

import com.couchbase.client.deps.io.netty.util.internal.logging.InternalLogger;
import com.couchbase.client.deps.io.netty.util.internal.logging.InternalLoggerFactory;

import static com.couchbase.client.deps.io.netty.util.internal.ObjectUtil.checkNotNull;

/**
 * {@link GenericFutureListener} implementation which takes other {@link Future}s
 * and notifies them on completion.
 *
 * @param <V> the type of value returned by the future
 * @param <F> the type of future
 */
public class PromiseNotifier<V, F extends Future<V>> implements GenericFutureListener<F> {

    private static final InternalLogger logger = InternalLoggerFactory.getInstance(PromiseNotifier.class);
    private final Promise<? super V>[] promises;

    /**
     * Create a new instance.
     *
     * @param promises  the {@link Promise}s to notify once this {@link GenericFutureListener} is notified.
     */
    @SafeVarargs
    public PromiseNotifier(Promise<? super V>... promises) {
        checkNotNull(promises, "promises");
        for (Promise<? super V> promise: promises) {
            if (promise == null) {
                throw new IllegalArgumentException("promises contains null Promise");
            }
        }
        this.promises = promises.clone();
    }

    @Override
    public void operationComplete(F future) throws Exception {
        if (future.isSuccess()) {
            V result = future.get();
            for (Promise<? super V> p: promises) {
                if (!p.trySuccess(result)) {
                    logger.warn("Failed to mark a promise as success because it is done already: {}", p);
                }
            }
            return;
        }

        Throwable cause = future.cause();
        for (Promise<? super V> p: promises) {
            if (!p.tryFailure(cause)) {
                logger.warn("Failed to mark a promise as failure because it's done already: {}", p, cause);
            }
        }
    }
}

==> core-io-1.2.9-sources.jar: com/couchbase/client/deps/io/netty/util/concurrent/PromiseNotifier.java <==
/*
 * Copyright 2014 The Netty Project
 *
 * The Netty Project licenses this file to you under the Apache License,
 * version 2.0 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at:
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package com.couchbase.client.deps.io.netty.util.concurrent;

import com.couchbase.client.deps.io.netty.util.internal.logging.InternalLogger;
import com.couchbase.client.deps.io.netty.util.internal.logging.InternalLoggerFactory;

import static com.couchbase.client.deps.io.netty.util.internal.ObjectUtil.checkNotNull;

/**
 * {@link GenericFutureListener} implementation which takes other {@link Future}s
 * and notifies them on completion.
 *
 * @param <V> the type of value returned by the future
 * @param <F> the type of future
 */
public class PromiseNotifier<V, F extends Future<V>> implements GenericFutureListener<F> {

    private static final InternalLogger logger = InternalLoggerFactory.getInstance(PromiseNotifier.class);
    private final Promise<? super V>[] promises;

    /**
     * Create a new instance.
     *
     * @param promises  the {@link Promise}s to notify once this {@link GenericFutureListener} is notified.
     */
    @SafeVarargs
    public PromiseNotifier(Promise<? super V>... promises) {
        checkNotNull(promises, "promises");
        for (Promise<? super V> promise: promises) {
            if (promise == null) {
                throw new IllegalArgumentException("promises contains null Promise");
            }
        }
        this.promises = promises.clone();
    }

    @Override
    public void operationComplete(F future) throws Exception {
        if (future.isSuccess()) {
            V result = future.get();
            for (Promise<? super V> p: promises) {
                if (!p.trySuccess(result)) {
                    logger.warn("Failed to mark a promise as success because it is done already: {}", p);
                }
            }
        } else if (future.isCancelled()) {
            for (Promise<? super V> p: promises) {
                if (!p.cancel(false)) {
                    logger.warn("Failed to cancel a promise because it is done already: {}", p);
                }
            }
        } else {
            Throwable cause = future.cause();
            for (Promise<? super V> p: promises) {
                if (!p.tryFailure(cause)) {
                    logger.warn("Failed to mark a promise as failure because it's done already: {}", p, cause);
                }
            }
        }
    }
}
